<?php

/**
 * @file
 * Administration screens for the Fragments module.
 */

/**
 * Generates the fragment type editing form.
 */
function fragments_type_form($form, &$form_state, $fragment_type, $op = 'edit') {
  if ($op == 'clone') {
    $fragment_type->label .= ' (cloned)';
    $fragment_type->type = '';
  }

  $form['label'] = array(
    '#title' => t('Label'),
    '#type' => 'textfield',
    '#default_value' => $fragment_type->label,
    '#description' => t('The human-readable name of this fragment type.'),
    '#required' => TRUE,
    '#size' => 30,
  );

  // Machine-readable type name.
  $form['type'] = array(
    '#type' => 'machine_name',
    '#default_value' => isset($fragment_type->type) ? $fragment_type->type : '',
    '#maxlength' => 32,
    '#disabled' => $fragment_type->isLocked() && $op != 'clone',
    '#machine_name' => array(
      'exists' => 'fragments_types',
      'source' => array('label'),
    ),
    '#description' => t('A unique machine-readable name for this fragment type. It must only contain lowercase letters, numbers, and underscores.'),
  );

  $form['description'] = array(
    '#type' => 'textarea',
    '#default_value' => isset($fragment_type->description) ? $fragment_type->description : '',
    '#description' => t('Describe this fragment type. The text will be displayed on the <em>Add new fragment</em> page.'),
  );

  $form['actions'] = array('#type' => 'actions');
  $form['actions']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save fragment type'),
    '#weight' => 40,
  );

  if (!$fragment_type->isLocked() && $op != 'add' && $op != 'clone') {
    $form['actions']['delete'] = array(
      '#type' => 'submit',
      '#value' => t('Delete fragment type'),
      '#weight' => 45,
      '#limit_validation_errors' => array(),
      '#submit' => array('fragments_type_form_submit_delete'),
    );
  }
  return $form;
}

/**
 * Submit handler for creating/editing a fragment type.
 */
function fragments_type_form_submit(&$form, &$form_state) {
  $fragment_type = entity_ui_form_submit_build_entity($form, $form_state);
  // Save and go back.
  fragments_type_save($fragment_type);

  // Redirect user back to list of fragment types.
  $form_state['redirect'] = 'admin/structure/fragment-types';
}

/**
 * Submit handler for deleting a fragment type.
 */
function fragments_type_form_submit_delete(&$form, &$form_state) {
  $form_state['redirect'] = 'admin/structure/fragment-types/' . $form_state['fragments_type']->type . '/delete';
}

/**
 * Fragment type delete form.
 */
function fragments_type_form_delete_confirm($form, &$form_state, $fragment_type) {
  $form_state['fragment_type'] = $fragment_type;
  // Always provide entity id in the same form key as in the entity edit form.
  $form['fragment_type_id'] = array(
    '#type' => 'value',
    '#value' => entity_id('fragments_type', $fragment_type),
  );
  return confirm_form($form,
    t(
      'Are you sure you want to delete fragment type %title?',
      array('%title' => entity_label('fragments_type', $fragment_type))
    ),
    'fragment/' . entity_id('fragments_type', $fragment_type),
    t('This action cannot be undone.'),
    t('Delete')
  );
}

/**
 * Fragment type delete form submit handler.
 */
function fragments_type_form_delete_confirm_submit($form, &$form_state) {
  $fragment_type = $form_state['fragment_type'];
  fragments_type_delete($fragment_type);

  watchdog('fragments', 'Type %title (@type) deleted.', array('@type' => $fragment_type->type, '%title' => $fragment_type->label));
  drupal_set_message(t('@type %title has been deleted.', array('@type' => $fragment_type->type, '%title' => $fragment_type->label)));

  $form_state['redirect'] = 'admin/structure/fragment-types';
}

/**
 * Page to select fragment type to add new fragment.
 */
function fragments_admin_add_page() {
  $item = menu_get_item();
  $content = system_admin_menu_block($item);
  // Bypass the fragment/add listing if only one content type is available.
  if (count($content) == 1) {
    $item = array_shift($content);

    // Prevent any destination value to short-circuit this, unset it here and
    // pass it along.
    $options = array();
    if (isset($_GET['destination'])) {
      $options['query']['destination'] = $_GET['destination'];
      unset($_GET['destination']);
    }
    drupal_goto($item['href'], $options);
  }

  return theme('fragments_add_list', array('content' => $content));
}

/**
 * Add new fragment page callback.
 */
function fragments_add($type) {
  $fragment_type = fragments_types($type);

  $fragment = entity_create('fragments_item', array('type' => $type));
  drupal_set_title(
    t(
      'Create @name',
      array('@name' => entity_label('fragments_type', $fragment_type))
    )
  );

  $output = drupal_get_form('fragments_form', $fragment);

  return $output;
}

/**
 * Fragment form.
 */
function fragments_form($form, &$form_state, $fragment) {
  $form_state['fragment'] = $fragment;

  $form['title'] = array(
    '#type' => 'textfield',
    '#required' => TRUE,
    '#title' => t('Title'),
    '#default_value' => $fragment->title,
  );

  $form['uid'] = array(
    '#type' => 'value',
    '#value' => $fragment->uid,
  );

  // Prevent redirects to be added. Redirect module's implementation of
  // hook_field_attach_form() contains a check whether the form key 'redirect'
  // is empty. If not, it returns.
  $form['redirect'] = array('#access' => FALSE);

  field_attach_form('fragments_item', $fragment, $form, $form_state);

  $form['status'] = array(
    '#type' => 'checkbox',
    '#title' => t('Published'),
    '#default_value' => $fragment->status,
    '#weight' => 99,
  );

  $submit = array();
  if (!empty($form['#submit'])) {
    $submit += $form['#submit'];
  }

  $form['actions'] = array(
    '#weight' => 100,
    '#prefix' => '<p>',
    '#suffix' => '</p>',
  );

  $form['actions']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save fragment'),
    '#submit' => $submit + array('fragments_form_submit'),
  );

  $form['actions']['cancel'] = array(
    '#type' => 'link',
    '#title' => t('Cancel'),
    '#weight' => 45,
    '#href' => url('admin/content/fragments'),
  );

  return $form;
}

/**
 * Submit handler for adding/editing a fragment item.
 */
function fragments_form_submit($form, &$form_state) {
  $fragment = $form_state['fragment'];

  entity_form_submit_build_entity('fragments_item', $fragment, $form, $form_state);

  fragments_save($fragment);

  $fragment_uri = entity_uri('fragments_item', $fragment);

  $form_state['redirect'] = $fragment_uri['path'];

  drupal_set_message(t('Fragment %title saved.', array('%title' => entity_label('fragments_item', $fragment))));
}

/**
 * Delete confirmation form.
 */
function fragments_delete_form($form, &$form_state, $fragment) {
  $form_state['fragment'] = $fragment;

  // Always provide entity id in the same form key as in the entity edit form.
  $form['fragment_type_id'] = array(
    '#type' => 'value',
    '#value' => entity_id('fragments_item', $fragment),
  );
  $fragment_id = entity_uri('fragments_item', $fragment);
  return confirm_form($form,
    t('Are you sure you want to delete fragment %title?', array('%title' => entity_label('fragments_item', $fragment))),
    $fragment_id['path'],
    t('This action cannot be undone.'),
    t('Delete')
  );
}

/**
 * Delete form submit handler.
 */
function fragments_delete_form_submit($form, &$form_state) {
  $fragment = $form_state['fragment'];
  fragments_delete($fragment);

  drupal_set_message(t('Fragment %title deleted.', array('%title' => entity_label('fragments_item', $fragment))));

  $form_state['redirect'] = '<front>';
}

/**
 * Returns HTML for a list of available fragment types for fragment creation.
 *
 * This mimicks the HTML for the node/add page from Seven.
 *
 * @param array $variables
 *   An associative array containing:
 *   - content: An array of fragment types.
 *
 * @return string
 *   A string of rendered HTML.
 */
function theme_fragments_add_list(array $variables) {
  $content = $variables['content'];
  $output = '';

  if ($content) {
    $output = '<ul class="admin-list">';
    foreach ($content as $item) {
      $output .= '<li class="clearfix">';
      $output .= '<span class="label">' . l($item['title'], $item['href'], $item['localized_options']) . '</span>';
      $output .= '<div class="description">' . filter_xss_admin($item['description']) . '</div>';
      $output .= '</li>';
    }
    $output .= '</ul>';
  }
  else {
    $output = '<p>' . t('You have not created any fragment types yet. Go to the <a href="@create-fragment">fragment type creation page</a> to add a new fragment type.', array('@create-content' => url('admin/structure/fragment-types/add'))) . '</p>';
  }
  return $output;
}
